/**
 * JASMINe
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.ow2.jasmine.jade.legacy.apache;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.net.URL;
import java.util.Dictionary;
import java.util.Hashtable;

import org.objectweb.jasmine.jade.util.ExecutableCmd;
import org.objectweb.jasmine.jade.util.FileCmd;
import org.objectweb.jasmine.jade.util.JadeException;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.Constants;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.ServiceRegistration;
import org.osgi.service.cm.ConfigurationException;
import org.osgi.service.cm.ManagedService;
import org.osgi.service.log.LogService;

public class Activator implements BundleActivator, ManagedService {

    /**
     *
     */
    private static final String PID = "org.ow2.jasmine.jade.wrapper.apache";

    /**
     *
     */
    private Dictionary properties = null;

    /**
     *
     */
    private ServiceRegistration registration = null;

    /**
     *
     */
    private BundleContext context = null;

    /**
     *
     */
    private LogService logger = null;

    /**
     *
     */
    private String dirLocal = null;

    /**
     *
     */
    private String dirInstall = null;

    /**
     *
     */
    private static final String APACHE_DIR = "apache2" + File.separator;

    /**
     *
     */
    private static final String APACHE_ARCHIVE = "apache-2.2.2.tgz";

    /**
     *
     */
    private boolean started = false;

    // ------------------------------------------------------------------------
    // Implementation of the BundleActivator interface
    // ------------------------------------------------------------------------

    /**
     * @see BundleActivator#start(BundleContext)
     */
    public void start(BundleContext context) throws Exception {

        if (!started) {

            registration = context.registerService(ManagedService.class
                    .getName(), this, getDefaults());

            this.context = context;

            ServiceReference sr = context.getServiceReference(LogService.class
                    .getName());
            logger = (LogService) context.getService(sr);
        }
    }

    /**
     * @see BundleActivator#stop(BundleContext)
     */
    public void stop(BundleContext context) throws Exception {
        removeApache();
        started = false;
    }

    // ------------------------------------------------------------------------
    // Implementation of the ManagedService interface
    // ------------------------------------------------------------------------

    public void updated(Dictionary properties) throws ConfigurationException {

        if (properties != null) {

            this.properties = properties;

            try {

                if (!started) {

                    dirLocal = getDirLocal();
                    dirInstall = dirLocal + APACHE_DIR;
                    started = install();
                }

            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    // ------------------------------------------------------------------------
    // Private methods
    // ------------------------------------------------------------------------

    /**
     * @return
     */
    private String getDirLocal() {

        String result = (String) properties.get("local.dir");

        if (!result.endsWith(File.separator)) {
            result += File.separator;
        }

        return result;
    }

    /**
     * @return
     */
    private Dictionary getDefaults() {

        Dictionary defaults = new Hashtable();

        defaults.put(Constants.SERVICE_PID, PID);
        // defaults.put("local.dir", System.getProperty("java.io.tmpdir"));

        return defaults;

    }

    /**
     * Installs on the local machine the Apache HTTP server from a TGZ file
     * contained in this bundle.
     *
     * @return true if the installation is successfull, false otherwise
     */
    private boolean install() {

        logger.log(LogService.LOG_INFO,
                "The apache directory is localised on : " + dirLocal);

        try {
            File dir = new File(dirLocal);
            if (!dir.exists()) {
                ExecutableCmd.syncExec("mkdir " + dirLocal, null);
            }

            File apacheArchive = new File(dirLocal + APACHE_ARCHIVE);

            /*
             * If APACHE_ARCHIVE already exists, doesn't dl it. Copy archive
             */
            if (!apacheArchive.exists()) {
                logger.log(LogService.LOG_INFO, "copying the '"
                        + APACHE_ARCHIVE + "' file to the local dir...");

                URL url = context.getBundle().getResource(APACHE_ARCHIVE);
                logger.log(LogService.LOG_INFO, context.getBundle()
                        .getLocation());
                logger.log(LogService.LOG_INFO, url.getFile());
                BufferedInputStream fis = new BufferedInputStream(url
                        .openConnection().getInputStream());
                FileOutputStream out = new FileOutputStream(apacheArchive);
                // ByteArrayInputStream out =new FileInputStream(url
                // .openConnection().getInputStream().);
                logger.log(LogService.LOG_INFO, "available : "
                        + fis.available());

                while (fis.available() > 0) {
                    byte[] byts = new byte[fis.available()];
                    fis.read(byts);
                    out.write(byts);
                }
                out.flush();
                fis.close();
                out.close();
                logger.log(LogService.LOG_INFO, "done");
            }

            /*
             * Extract archive
             */
            Runtime r = Runtime.getRuntime();
            logger.log(LogService.LOG_INFO, "extracting the '" + APACHE_ARCHIVE
                    + "' in the local dir...");
            logger.log(LogService.LOG_INFO, "tar xf " + dirLocal
                    + APACHE_ARCHIVE + " --directory=" + dirLocal);
            Process p = r.exec("tar xf " + dirLocal + APACHE_ARCHIVE
                    + " --directory=" + dirLocal);
            p.waitFor();
            logger.log(LogService.LOG_INFO, "done");

            apacheArchive.delete();

            return true;

        } catch (IOException e) {
            logger.log(LogService.LOG_ERROR, "failed", e);
            return false;
        } catch (JadeException e) {
            logger.log(LogService.LOG_ERROR, "failed", e);
            return false;
        } catch (InterruptedException e) {
            logger.log(LogService.LOG_ERROR, "failed", e);
            return false;
        }
    }

    /**
     * Removes the Apache HTTP server from the local machine.
     */
    private void removeApache() {

        String filename = dirInstall + "logs" + File.separator + "httpd.pid";

        logger.log(LogService.LOG_INFO, "stopping the Apache HTTP server...");

        try {

            FileCmd.assertFileExists(filename);

            String s = null;
            try {
                BufferedReader br = new BufferedReader(new FileReader(filename));
                s = br.readLine();

            } catch (IOException e) {
                logger.log(LogService.LOG_ERROR,
                        "Error during read of the Apache pid file", e);
            }

            /*
             * Stop server
             */
            try {

                ExecutableCmd.syncExec(dirInstall + "bin" + File.separator
                        + "apachectl stop", null);

                ExecutableCmd.asyncExec("kill -TERM " + s, null);
                logger.log(LogService.LOG_INFO, "done");

            } catch (Exception e) {
                logger.log(LogService.LOG_ERROR,
                        "Error while stopping the Apache HTTP server", e);
            }

        } catch (JadeException e) {
            logger.log(LogService.LOG_ERROR,
                    "Error while stopping the Apache HTTP server", e);

        } finally {

            /*
             * remove directory
             */
            logger.log(LogService.LOG_INFO, "deleting apache directory...");

            try {

                ExecutableCmd.syncExec("rm -r " + dirInstall, null);
                logger.log(LogService.LOG_INFO, "done");

            } catch (JadeException e) {
                logger.log(LogService.LOG_ERROR,
                        "Error while erasing the Apache HTTP server directory",
                        e);
            }
        }
    }
}
